@@ -169,6 +169,8 @@ sub Create {
         return ( 0, $self->loc("No permission to create queues") );
     }
 
+    $args{'Name'} = $self->CanonicalizeName( $args{'Name'} );
+
     {
         my ($val, $msg) = $self->_ValidateName( $args{'Name'} );
         return ($val, $msg) unless $val;
@@ -280,7 +282,7 @@ sub _ValidateName {
     $tempqueue->Load($name);
 
     #If this queue exists, return undef
-    if ( $tempqueue->Name() && $tempqueue->id != $self->id)  {
+    if ( $tempqueue->Name() && ( !$self->id || $tempqueue->id != $self->id ) ) {
         return (undef, $self->loc("Queue already exists") );
     }
 
@@ -481,6 +483,7 @@ sub CustomRoles {
     my $roles = RT::CustomRoles->new( $self->CurrentUser );
     if ( $self->CurrentUserHasRight('SeeQueue') ) {
         $roles->LimitToObjectId( $self->Id );
+        $roles->LimitToLookupType(RT::Ticket->CustomFieldLookupType);
         $roles->ApplySortOrder;
     }
     else {
@@ -809,6 +812,29 @@ sub CurrentUserCanSee {
     return $self->CurrentUserHasRight('SeeQueue');
 }
 
+
+=head2 CurrentUserCanCreate
+
+Returns true if the current user can create a new queue, using I<AdminQueue>.
+
+=cut
+
+sub CurrentUserCanCreate {
+    my $self = shift;
+    return $self->CurrentUserHasRight('AdminQueue');
+}
+
+=head2 CurrentUserCanModify
+
+Returns true if the current user can modify the queue, using I<AdminQueue>.
+
+=cut
+
+sub CurrentUserCanModify {
+    my $self = shift;
+    return $self->CurrentUserHasRight('AdminQueue');
+}
+
 =head2 id
 
 Returns the current value of id. 
@@ -835,6 +861,13 @@ Returns (1, 'Status message') on success and (0, 'Error Message') on failure.
 
 =cut
 
+sub SetName {
+    my $self = shift;
+    my $value = shift;
+
+    my ($status, $msg) = $self->_Set( Field => 'Name', Value => $self->CanonicalizeName($value) );
+    return ($status, $msg);
+}
 
 =head2 Description
 
@@ -1075,15 +1108,18 @@ sub FindDependencies {
                   VALUE    => 'RT::Queue-' );
     $deps->Add( in => $objs );
 
-    # Tickets
-    $objs = RT::Tickets->new( $self->CurrentUser );
-    $objs->Limit( FIELD => "Queue", VALUE => $self->Id );
-    $objs->{allow_deleted_search} = 1;
-    $deps->Add( in => $objs );
+    # Tickets (skipped early as an optimization)
+    if ($walker->{FollowTickets} || !defined($walker->{FollowTickets})) {
+        $objs = RT::Tickets->new( $self->CurrentUser );
+        $objs->Limit( FIELD => "Queue", VALUE => $self->Id );
+        $objs->{allow_deleted_search} = 1;
+        $deps->Add( in => $objs );
+    }
 
     # Object Custom Roles
     $objs = RT::ObjectCustomRoles->new( $self->CurrentUser );
     $objs->LimitToObjectId($self->Id);
+    $objs->LimitToLookupType(RT::Ticket->CustomFieldLookupType);
     $deps->Add( in => $objs );
 }
 
@@ -1125,6 +1161,7 @@ sub __DependsOn {
 
 # Object Custom Roles
     $objs = RT::ObjectCustomRoles->new( $self->CurrentUser );
+    $objs->LimitToLookupType( RT::Ticket->CustomFieldLookupType );
     $objs->LimitToObjectId($self->Id);
     push( @$list, $objs );
 
@@ -1156,8 +1193,32 @@ sub DefaultValue {
     my $self = shift;
     my $field = shift;
     my $attr = $self->FirstAttribute('DefaultValues');
-    return undef unless $attr && $attr->Content;
-    return $attr->Content->{$field};
+
+    my $fallback;
+    if ( $field =~ /Priority/
+        && RT->Config->Get('EnablePriorityAsString') )
+    {
+        my %config = RT->Config->Get('PriorityAsString');
+        my $queue_name = $self->__Value('Name');
+        if ( my $value = exists $config{$queue_name} ? $config{$queue_name} : $config{Default} ) {
+
+            # For ordered list(array), use the first entry. For unordered list(hash), use the lowest value.
+            # This is also consistent with the priority order on web pages
+            if ( ref $value eq 'ARRAY' ) {
+                $fallback = $value->[1];
+            }
+            elsif ( ref $value eq 'HASH' ) {
+                $fallback = ( sort { $a <=> $b } values %$value )[0];
+            }
+            else {
+                RT->Logger->warning("Invalid PriorityAsString value: $value");
+            }
+        }
+    }
+
+    return $fallback unless $attr && $attr->Content;
+    my $value = $attr->Content->{$field};
+    return $value // $fallback;
 }
 
 sub SetDefaultValue {
@@ -1185,6 +1246,29 @@ sub SetDefaultValue {
         $new_value = $self->loc( '(no value)' );
     }
 
+    if ( $args{Name} eq 'Article' && $args{Value} ) {
+        my $article = RT::Article->new($self->CurrentUser);
+        my ($ret, $msg);
+        if ( $args{Value} =~ /^\d+$/ ) {
+            ($ret, $msg) = $article->Load( $args{Value} );
+        }
+        else {
+            ($ret, $msg) = $article->LoadByCols( Name => $args{Value} );
+        }
+        return ($ret, $msg ) unless $ret;
+
+        $args{Value} = $article->Id;
+        $new_value = $article->Name;
+    }
+
+    if ( $args{Name} eq 'Article' && $old_value =~ /^\d+$/ ) {
+        my $article = RT::Article->new($self->CurrentUser);
+        my ($ret, $msg) = $article->Load( $old_value );
+        if ($ret) {
+            $old_value = $article->Name;
+        }
+    }
+
     return 1 if $new_value eq $old_value;
 
     my ($ret, $msg) = $self->SetAttribute(
@@ -1194,6 +1278,17 @@ sub SetDefaultValue {
         },
     );
 
+    if ( $args{Name} =~ /Priority/ && RT->Config->Get('EnablePriorityAsString') ) {
+        if ( $old_value ne $self->loc('(no value)') ) {
+            my $str = RT::Ticket->_PriorityAsString( $old_value, $self->Name );
+            $old_value = $self->loc($str) if $str;
+        }
+        if ( $new_value ne $self->loc('(no value)') ) {
+            my $str = RT::Ticket->_PriorityAsString( $new_value, $self->Name );
+            $new_value = $self->loc($str) if $str;
+        }
+    }
+
     if ( $ret ) {
         return ( $ret, $self->loc( 'Default value of [_1] changed from [_2] to [_3]', $args{Name}, $old_value, $new_value ) );
     }
@@ -1289,6 +1384,35 @@ sub SetDefaultDueIn {
     );
 }
 
+sub HiddenCustomRoleIDsForURL {
+    my $self = shift;
+    my $url  = shift;
+
+    my $roles = $self->CustomRoles;
+    my @ids;
+
+    while (my $role = $roles->Next) {
+        push @ids, $role->Id if $role->IsHiddenForURL($url);
+    }
+
+    return @ids;
+}
+
+
+=head2 CanonicalizeName NAME
+
+Strip leading/trailing spaces and returns the updated name.
+
+=cut
+
+sub CanonicalizeName {
+    my $self = shift;
+    my $name = shift // return undef;
+    $name =~ s!^\s+!!;
+    $name =~ s!\s+$!!;
+    return $name;
+}
+
 RT::Base->_ImportOverlays();
 
 1;
